@@ -154,7 +154,7 @@ CMPIValue *getKeyValueTypePtr(char *type, char *value, XtokValueReference *ref,
       else if (strcasecmp(type, "ref") == 0) {
          CMPIObjectPath *op;
          char *hn="",*ns="",*cn;
-         CMPIType type;
+	 CMPIType type;
          CMPIValue v, *valp;
          int i,m;
          XtokInstanceName *in; 
@@ -175,7 +175,7 @@ CMPIValue *getKeyValueTypePtr(char *type, char *value, XtokValueReference *ref,
             break;   
          default:
             mlogf(M_ERROR,M_SHOW,"%s(%d): unexpected reference type %d %x\n", __FILE__, __LINE__, 
-               (int) type, (int) type);
+               (int) ref->type, (int) ref->type);
             abort();   
          }
 
@@ -320,7 +320,7 @@ CMPIValue makeFromEmbeddedObject(XtokValue value, char *ns)
 CMPIValue str2CMPIValue(CMPIType type, XtokValue val, XtokValueReference *ref, char* ns)
 {
    CMPIValue value;
-   CMPIType t;
+   CMPIType t = 0;
 
    if (type==0) {
       type=guessType(val.value);
@@ -535,7 +535,9 @@ static int lnsPath2xml(CMPIObjectPath * ci, UtilStringBuffer * sb)
        SFCB_APPENDCHARS_BLOCK(sb, "<NAMESPACE NAME=\"");
        sb->ft->appendChars(sb, ns);
        SFCB_APPENDCHARS_BLOCK(sb, "\"/>\n");
-       ns = nsp + 1;
+       if (nsp) {
+	 ns = nsp + 1;
+       }
      } while (nsp);
      free(nsc);
      SFCB_APPENDCHARS_BLOCK(sb, "</LOCALNAMESPACEPATH>\n");  
@@ -556,9 +558,10 @@ static int nsPath2xml(CMPIObjectPath * ci, UtilStringBuffer * sb)
       hn = malloc(HOST_NAME_MAX);
       if(gethostname(hn, HOST_NAME_MAX) == 0) {
         sb->ft->appendChars(sb, hn);
-      } else {
+       } else {
         SFCB_APPENDCHARS_BLOCK(sb, "localhost");
       }
+      free (hn);
    }
    SFCB_APPENDCHARS_BLOCK(sb, "</HOST>\n");
    
@@ -801,7 +804,10 @@ static void param2xml(CMPIParameter *pdata, CMPIConstClass * cls, ClParameter *p
       SFCB_APPENDCHARS_BLOCK(sb, "\">\n");
    } 
  
-   if (qsb) sb->ft->appendChars(sb, (char *) qsb->hdl);
+   if (qsb) {
+     sb->ft->appendChars(sb, (char *) qsb->hdl);
+     qsb->ft->release(qsb);
+   }
    sb->ft->appendChars(sb, etag);
 }
 
@@ -850,16 +856,7 @@ int cls2xml(CMPIConstClass * cls, UtilStringBuffer * sb, unsigned int flags)
       else {
          type = dataType(data.type);
          if (*type == '*') {
-            if (data.state &CMPI_nullValue) {
-               SFCB_APPENDCHARS_BLOCK(sb, "<PROPERTY.REFERENCE NAME=\"");
-               sb->ft->appendChars(sb, (char*)name->hdl);
-               if(refName) {
-                  SFCB_APPENDCHARS_BLOCK(sb, "\" REFERENCECLASS=\"");
-                  sb->ft->appendChars(sb, (char*)refName->hdl);
-               }
-               SFCB_APPENDCHARS_BLOCK(sb, "\"></PROPERTY.REFERENCE>\n");
-            }
-            else DATA2XML(&data,cls,name,refName,"<PROPERTY.REFERENCE NAME=\"",
+            DATA2XML(&data,cls,name,refName,"<PROPERTY.REFERENCE NAME=\"",
                    "</PROPERTY.REFERENCE>\n", sb, qsb, 0,0);
          }   
          else  DATA2XML(&data,cls,name,NULL,"<PROPERTY NAME=\"", "</PROPERTY>\n",
