@@ -1,6 +1,6 @@
 
 /*
- * $Id: cimXmlRequest.c,v 1.55 2009/06/05 19:18:24 buccella Exp $
+ * $Id: cimXmlRequest.c,v 1.58 2010/03/29 23:42:31 smswehla Exp $
  *
  * © Copyright IBM Corp. 2005, 2007
  *
@@ -1031,6 +1031,7 @@ static RespSegments enumClasses(CimXmlRequestContext * ctx,
 	freeResponseHeaders(resp,&binCtx);
 	_SFCB_RETURN(rs);
       }
+      freeResponseHeaders(resp,&binCtx);
       
       rs.chunkedMode=1;
       rs.rc=err;
@@ -2105,6 +2106,17 @@ static RespSegments invokeMethod(CimXmlRequestContext * ctx, RequestHdr * hdr)
    _SFCB_TRACE(1, ("--- Getting Provider context"));
    irc = getProviderContext(&binCtx, (OperationHdr *) req);
 
+   if (irc == MSG_X_SFCB_PROVIDER) {
+     if(*req->method == '_') {
+       RespSegments  rs;
+       rs = methodErrResponse(hdr, getErrSegment(CMPI_RC_ERR_ACCESS_DENIED, NULL));
+       closeProviderContext(&binCtx);
+       _SFCB_RETURN(rs);
+     } else {
+       irc = MSG_X_PROVIDER;
+     }
+   }
+                                                 
    _SFCB_TRACE(1, ("--- Provider context gotten"));
    if (irc == MSG_X_PROVIDER) {
       RespSegments rs;
@@ -2207,8 +2219,10 @@ static RespSegments getProperty(CimXmlRequestContext * ctx, RequestHdr * hdr)
       if (resp) {
     free(resp);
       }
+      CMRelease(tmpString);
       _SFCB_RETURN(rs);
    }
+   CMRelease(tmpString);
    closeProviderContext(&binCtx);
 
    _SFCB_RETURN(ctxErrResponse(hdr, &binCtx,0));
