@@ -1,6 +1,6 @@
 
 /*
- * $Id: cimcClientSfcbLocal.c,v 1.29 2009/03/31 21:29:46 buccella Exp $
+ * $Id: cimcClientSfcbLocal.c,v 1.35 2010/04/07 20:54:10 buccella Exp $
  *
  * © Copyright IBM Corp. 2006, 2007
  *
@@ -23,6 +23,8 @@
 #include "cmpidt.h"
 #include "cmpift.h"
 #include "cmpimacs.h"
+#include "mlog.h"
+#include <syslog.h>
 
 #include "providerMgr.h"
 #include "queryOperation.h"
@@ -207,6 +209,7 @@ static CMPIStatus releaseClient(Client * mb)
   if (cl->connection) CMRelease(cl->connection);
 
   free(cl);
+  closeLogging();
   return rc;
 }
 
@@ -811,16 +814,19 @@ static CMPIEnumeration * execQuery(
       if (err == 0) {
          enm=cpyEnumResponses(&binCtx,resp,l);
          freeResps(resp,binCtx.pCount);
+         CMRelease(path);
          _SFCB_RETURN(enm);
       }
       if (rc) CIMCSetStatusWithChars(rc, resp[err-1]->rc, 
                   (char*)resp[err-1]->object[0].data);
+      CMRelease(path);
       freeResps(resp,binCtx.pCount);
       _SFCB_RETURN(NULL);
    }
    else ctxErrResponse(&binCtx,rc);
    closeProviderContext(&binCtx);
-    
+
+   CMRelease(path);
    _SFCB_RETURN(NULL);
 }	
 	
@@ -1216,8 +1222,9 @@ static CMPIData invokeMethod(
             CMAddArg(out,(char*)name->hdl,&data.value,data.type);
          }
          if (resp->rvValue) {
+	    /* check method return value for pass-by-reference types */
             if (resp->rv.type==CMPI_chars) {
-               resp->rv.value.chars=(long)resp->rvEnc.data+(char*)resp;
+               resp->rv.value.chars=strdup((long)resp->rvEnc.data+(char*)resp);
             }
             else if (resp->rv.type==CMPI_dateTime) {
                resp->rv.value.dateTime=
@@ -1643,10 +1650,14 @@ static Client *CMPIConnect2(ClientEnv* ce, const char *hn, const char *scheme, c
 			 const char * certFile, const char * keyFile,
 			 CMPIStatus *rc)
 {  
-   ClientEnc *cc = (ClientEnc*)calloc(1, sizeof(ClientEnc));
+   ClientEnc *cc;
 
-   if (rc) rc->rc=0;
+   if (rc) CMSetStatus(rc, 0);
    
+   if (localConnect(ce,rc)<0) return NULL;
+
+   cc = (ClientEnc*)calloc(1, sizeof(ClientEnc));
+
    cc->enc.hdl		= &cc->data;
    cc->enc.ft		= &clientFt;
 
@@ -1666,8 +1677,6 @@ static Client *CMPIConnect2(ClientEnv* ce, const char *hn, const char *scheme, c
    cc->certData.certFile = certFile ? strdup(certFile) : NULL;
    cc->certData.keyFile = keyFile ? strdup(keyFile) : NULL;
 
-   if (localConnect(ce,rc)<0) return NULL;
-
    return (Client*)cc;
 }
 
@@ -1736,6 +1745,8 @@ ClientEnv* _Create_SfcbLocal_Env(char *id)
     ClientEnv *env = (ClientEnv*)malloc(sizeof(ClientEnv));
     env->hdl=NULL;
     env->ft=&localFT;
+    // enable logging when called from sfcc
+    startLogging(LOG_ERR);
     
     return env;
  }
