@@ -18,39 +18,10 @@
  *
 */
 
-
-
-#include "utilft.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <string.h>
-#include <sys/types.h>
-#include <dirent.h>
-
-//#include "classRegister.h"
-#include "constClass.h"
-#include "providerRegister.h"
-#include "trace.h"
-#include "control.h"
-
-#define NEW(x) ((x *) malloc(sizeof(x)))
-
-#include "cmpidt.h"
-#include "cmpift.h"
-#include "cmpiftx.h"
-#include "cmpimacs.h"
-#include "cmpimacsx.h"
-#include "objectImpl.h"
-#include "mrwlock.h"
+#include "classProviderCommon.h"
 
 #define LOCALCLASSNAME "ClassProvider"
 
-static const CMPIBroker *_broker;
-
-extern char * configfile;
-extern ProviderRegister *pReg;
-
 typedef struct _Class_Register_FT Class_Register_FT;
 struct _ClassRegister {
    void *hdl;
@@ -85,14 +56,8 @@ extern Class_Register_FT *ClassRegisterFT;
 
 
 static CMPIConstClass *getClass(ClassRegister * cr, const char *clsName);
+int traverseChildren(ClassRegister *cReg, const char *parent, const char *child);
 
-typedef struct nameSpaces {
-   int next,max,blen;
-   char *base;
-   char *names[1];
-} NameSpaces;
-
-static UtilHashTable *nsHt=NULL;
 static int nsBaseLen;
 
 static void buildInheritanceTable(ClassRegister * cr)
@@ -454,31 +419,33 @@ static UtilHashTable *gatherNameSpaces(char *dn, UtilHashTable *ns, int first)
    }          
        
    dir=opendir(dn);
-   if (dir) while ((de=readdir(dir))!=NULL) {
-     if (strcmp(de->d_name,".")==0) continue;
-     if (strcmp(de->d_name,"..")==0) continue;
-     l=strlen(dn)+strlen(de->d_name)+4;
-     n=(char*)malloc(l+8);
-     strcpy(n,dn);
-     strcat(n,"/");
-     strcat(n,de->d_name);
-     dir_test = opendir(n);
-     if (dir_test == NULL) {
+   if (dir) {
+     while ((de=readdir(dir))!=NULL) {
+       if (strcmp(de->d_name,".")==0) continue;
+       if (strcmp(de->d_name,"..")==0) continue;
+       l=strlen(dn)+strlen(de->d_name)+4;
+       n=(char*)malloc(l+8);
+       strcpy(n,dn);
+       strcat(n,"/");
+       strcat(n,de->d_name);
+       dir_test = opendir(n);
+       if (dir_test == NULL) {
+         free(n);
+         continue;
+       }
+       closedir(dir_test);
+       cr=newClassRegister(n);
+       if (cr) {
+         ns->ft->put(ns, strdup(n+nsBaseLen), cr);
+         gatherNameSpaces(n,ns,0);
+       }
        free(n);
-       continue;
      }
-     closedir(dir_test);
-     cr=newClassRegister(n);
-     if (cr) {
-       ns->ft->put(ns, strdup(n+nsBaseLen), cr);
-       gatherNameSpaces(n,ns,0);
-     }
-     free(n);
+     closedir(dir);  
    }
-   else if (first) {
+   else if (first || dir==NULL) {
       mlogf(M_ERROR,M_SHOW,"--- Repository %s not found\n",dn);
    }
-   closedir(dir);  
    return ns;     
 } 
 
@@ -500,6 +467,11 @@ static UtilHashTable *buildClassRegisters()
    return gatherNameSpaces(dn,NULL,1);   
 }    
 
+static void nsHt_init()
+{
+  nsHt=buildClassRegisters();
+}
+
 
 static ClassRegister *getNsReg(const CMPIObjectPath *ref, int *rc)
 {
@@ -508,8 +480,14 @@ static ClassRegister *getNsReg(const CMPIObjectPath *ref, int *rc)
    ClassRegister *cReg;
    *rc=0;
    
-   if (nsHt==NULL) nsHt=buildClassRegisters();
-   
+   pthread_once(&nsHt_once, nsHt_init);
+
+   if (nsHt==NULL) {
+     mlogf(M_ERROR,M_SHOW,"--- ClassProvider: namespace hash table not initialized\n");
+     *rc = 1;
+     return NULL;
+   }
+
    if (nsi && nsi->hdl) {
       ns=(char*)nsi->hdl;
       if (strcasecmp(ns,"root/pg_interop")==0)
@@ -1108,22 +1086,8 @@ static CMPIStatus ClassProviderInvokeMethod(CMPIMethodMI * mi,
 
    else if (strcasecmp(methodName, "ischild") == 0) {
       char *parent=(char*)CMGetClassName(ref,NULL)->hdl;
-      UtilList *ul = getChildren(cReg, parent);
       char *chldn=(char*)CMGetArg(in, "child", NULL).value.string->hdl;
-      char *child;
-      
-      cReg->ft->rLock(cReg);
-   
-      st.rc = CMPI_RC_ERR_FAILED;
-      if (ul) for (child=(char*)ul->ft->getFirst(ul); child; 
-                   child=(char*)ul->ft->getNext(ul)) {
-         if (strcasecmp(child,chldn)==0 ) {
-            st.rc=CMPI_RC_OK;
-            break;
-         }   
-      }
-      
-      cReg->ft->rUnLock(cReg);
+      st.rc = traverseChildren(cReg, parent, chldn);
    }
    
    else if (strcasecmp(methodName, "_startup") == 0) {
@@ -1137,6 +1101,29 @@ static CMPIStatus ClassProviderInvokeMethod(CMPIMethodMI * mi,
    _SFCB_RETURN(st);
 }
 
+int traverseChildren(ClassRegister *cReg, const char *parent, const char *chldn) {
+      char *child;
+      int rc = CMPI_RC_ERR_FAILED;
+      UtilList *ul = getChildren(cReg, parent);
+      
+      cReg->ft->rLock(cReg);
+   
+      if (ul) for (child=(char*)ul->ft->getFirst(ul); child; 
+                   child=(char*)ul->ft->getNext(ul)) {
+         if (strcasecmp(child,chldn)==0 ) {
+            rc=CMPI_RC_OK;
+            break;
+         }
+         else {
+            rc = traverseChildren(cReg, child, chldn);
+            if(rc == CMPI_RC_OK) break;
+         }
+      }
+      
+      cReg->ft->rUnLock(cReg);
+      return rc;
+}
+   
 
 CMClassMIStub(ClassProvider, ClassProvider, _broker, CMNoHook);
 
