@@ -19,42 +19,12 @@
  *
 */
 
-
-
-#include "utilft.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <string.h>
-#include <sys/types.h>
-#include <dirent.h>
+#include "classProviderCommon.h"
 #include <zlib.h>
 
-//#include "classRegister.h"
-#include "constClass.h"
-#include "providerRegister.h"
-#include "trace.h"
-#include "control.h"
-
-#define NEW(x) ((x *) calloc(1,sizeof(x)))
-
-#include "cmpidt.h"
-#include "cmpift.h"
-#include "cmpiftx.h"
-#include "cmpimacs.h"
-#include "cmpimacsx.h"
-#include "objectImpl.h"
-#include "mrwlock.h"
-
 #define LOCALCLASSNAME "ClassProvider"
-#define Iterator HashTableIterator*
 
-static int cacheLimit=10;
-
-static const CMPIBroker *_broker;
-
-extern char * configfile;
-extern ProviderRegister *pReg;
+static unsigned int cacheLimit=10;
 
 typedef struct _Class_Register_FT Class_Register_FT;
 struct _ClassRegister {
@@ -116,15 +86,8 @@ extern Class_Register_FT *ClassRegisterFT;
 
 
 static CMPIConstClass *getClass(ClassRegister * cr, const char *clsName, void **id);
+int traverseChildren(ClassRegister *cReg, const char *parent, const char *child);
 
-typedef struct nameSpaces {
-   int next,max,blen;
-   char *base;
-   char *names[1];
-} NameSpaces;
-
-static UtilHashTable *nsHt=NULL;
-static pthread_once_t nsHt_once = PTHREAD_ONCE_INIT;
 static int nsBaseLen;
 
 static void buildInheritanceTable(ClassRegister * cr)
@@ -270,7 +233,6 @@ static void pruneCache(ClassRegister * cr)
 {
    ClassBase *cb = (ClassBase *) (cr + 1);
    ClassRecord *crec;
-   
    while (cb->cachedCount>cacheLimit) {
       crec=cb->lastCached;
 //    fprintf(stderr,"--- removing %s from cache\n",crec->cachedCls->ft->getCharClassName(crec->cachedCls));
@@ -756,8 +718,8 @@ static CMPIStatus ClassProviderEnumClasses(CMPIClassMI * mi,
            it = cReg->ft->getNextClass(cReg, it, &key, &cls,&cid)) {
          if ((flgs & CMPI_FLAG_DeepInheritance) || cls->ft->getCharSuperClassName(cls)==NULL) {  
             CMReturnInstance(rslt, (CMPIInstance *) cls);
-            if (cid==NULL) CMRelease(cls);
          }   
+ 	 if(cid == NULL) CMRelease(cls);
       }     
    } else {
      cls = getClass(cReg,cn,NULL);
@@ -1021,25 +983,25 @@ static CMPIStatus ClassProviderInvokeMethod(CMPIMethodMI * mi,
 
    else if (strcasecmp(methodName, "ischild") == 0) {
       char *parent=(char*)CMGetClassName(ref,NULL)->hdl;
-      UtilList *ul = getChildren(cReg, parent);
       char *chldn=(char*)CMGetArg(in, "child", NULL).value.string->hdl;
-      char *child;
-      
-      cReg->ft->wLock(cReg);
-   
-      st.rc = CMPI_RC_ERR_FAILED;
-      if (ul) for (child=(char*)ul->ft->getFirst(ul); child; 
-                   child=(char*)ul->ft->getNext(ul)) {
-         if (strcasecmp(child,chldn)==0 ) {
-            st.rc=CMPI_RC_OK;
-            break;
-         }   
-      }
-      
-      cReg->ft->wUnLock(cReg);
+      st.rc = traverseChildren(cReg, parent, chldn);
    }
    
    else if (strcasecmp(methodName, "_startup") == 0) {
+
+     /* check to see if cacheLimit was specified in providerRegister */
+     CMPIStatus parm_st = { CMPI_RC_OK, NULL };
+     CMPIData parmdata = CMGetContextEntry(ctx, "sfcbProviderParameters", &parm_st);
+
+     if (parm_st.rc == CMPI_RC_OK ) {
+       const char* parms = CMGetCharPtr(parmdata.value.string);
+
+       /* cacheLimit is currently the only param, so just take whatever is after the '=' */
+       const char* val = strchr(parms,'=');
+       /* conversion to uint may cause wrapping; won't catch negatives */
+       cacheLimit = ((val != NULL) && (cacheLimit = atoi(val+sizeof(char)))>0) ? cacheLimit : 10;
+     }
+
       st.rc=CMPI_RC_OK;
   }
    
@@ -1050,7 +1012,29 @@ static CMPIStatus ClassProviderInvokeMethod(CMPIMethodMI * mi,
    _SFCB_RETURN(st);
 }
 
-
+int traverseChildren(ClassRegister *cReg, const char *parent, const char *chldn) {
+      char *child;
+      int rc = CMPI_RC_ERR_FAILED;
+      UtilList *ul = getChildren(cReg, parent);
+      
+      cReg->ft->rLock(cReg);
+   
+      if (ul) for (child=(char*)ul->ft->getFirst(ul); child; 
+                   child=(char*)ul->ft->getNext(ul)) {
+         if (strcasecmp(child,chldn)==0 ) {
+            rc=CMPI_RC_OK;
+            break;
+         }
+         else {
+            rc = traverseChildren(cReg, child, chldn);
+            if(rc == CMPI_RC_OK) break;
+         }
+      }
+      
+      cReg->ft->rUnLock(cReg);
+      return rc;
+}
+   
 CMClassMIStub(ClassProvider, ClassProvider, _broker, CMNoHook);
 
 CMMethodMIStub(ClassProvider, ClassProvider, _broker, CMNoHook);
