@@ -1,6 +1,6 @@
 
 /*
- * $Id: classProviderMem.c,v 1.8 2009/04/27 18:40:01 mchasal Exp $
+ * $Id: classProviderMem.c,v 1.11 2009/12/17 20:15:48 buccella Exp $
  *
  * (C) Copyright IBM Corp. 2006
  *
@@ -20,39 +20,12 @@
 */
 
 
-
-#include "utilft.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <string.h>
-#include <sys/types.h>
-
-//#include "classRegister.h"
-#include "constClass.h"
-#include "providerRegister.h"
-#include "trace.h"
-#include "control.h"
-
-#define NEW(x) ((x *) malloc(sizeof(x)))
-
-#include "cmpidt.h"
-#include "cmpift.h"
-#include "cmpiftx.h"
-#include "cmpimacs.h"
-#include "cmpimacsx.h"
-#include "objectImpl.h"
-#include "mrwlock.h"
+#include "classProviderCommon.h"
 
 #define LOCALCLASSNAME "ClassProvider"
 
 #include "classSchemaMem.h"
 
-static const CMPIBroker *_broker;
-
-extern char * configfile;
-extern ProviderRegister *pReg;
-
 typedef struct _Class_Register_FT Class_Register_FT;
 struct _ClassRegister {
    void *hdl;
@@ -87,14 +60,7 @@ extern Class_Register_FT *ClassRegisterFT;
 
 
 static CMPIConstClass *getClass(ClassRegister * cr, const char *clsName);
-
-typedef struct nameSpaces {
-   int next,max,blen;
-   char *base;
-   char *names[1];
-} NameSpaces;
-
-static UtilHashTable *nsHt=NULL;
+int traverseChildren(ClassRegister *cReg, const char *parent, const char *child);
 
 static void buildInheritanceTable(ClassRegister * cr)
 {
@@ -400,6 +366,11 @@ static UtilHashTable *buildClassRegisters()
    return gatherNameSpaces();   
 }    
 
+static void nsHt_init()
+{
+  nsHt=buildClassRegisters();
+}
+
 
 static ClassRegister *getNsReg(const CMPIObjectPath *ref, int *rc)
 {
@@ -408,8 +379,14 @@ static ClassRegister *getNsReg(const CMPIObjectPath *ref, int *rc)
    ClassRegister *cReg;
    *rc=0;
    
-   if (nsHt==NULL) nsHt=buildClassRegisters();
-   
+   pthread_once(&nsHt_once, nsHt_init);
+
+   if (nsHt==NULL) {
+     mlogf(M_ERROR,M_SHOW,"--- ClassProvider: namespace hash table not initialized\n");
+     *rc = 1;
+     return NULL;
+   }
+
    if (nsi && nsi->hdl) {
       ns=(char*)nsi->hdl;
       if (strcasecmp(ns,"root/pg_interop")==0)
@@ -958,22 +935,8 @@ static CMPIStatus ClassProviderInvokeMethod(CMPIMethodMI * mi,
 
    else if (strcasecmp(methodName, "ischild") == 0) {
       char *parent=(char*)CMGetClassName(ref,NULL)->hdl;
-      UtilList *ul = getChildren(cReg, parent);
       char *chldn=(char*)CMGetArg(in, "child", NULL).value.string->hdl;
-      char *child;
-      
-      cReg->ft->rLock(cReg);
-   
-      st.rc = CMPI_RC_ERR_FAILED;
-      if (ul) for (child=(char*)ul->ft->getFirst(ul); child; 
-                   child=(char*)ul->ft->getNext(ul)) {
-         if (strcasecmp(child,chldn)==0 ) {
-            st.rc=CMPI_RC_OK;
-            break;
-         }   
-      }
-      
-      cReg->ft->rUnLock(cReg);
+      st.rc = traverseChildren(cReg, parent, chldn);
    }
    
    else if (strcasecmp(methodName, "_startup") == 0) {
@@ -987,7 +950,29 @@ static CMPIStatus ClassProviderInvokeMethod(CMPIMethodMI * mi,
    _SFCB_RETURN(st);
 }
 
-
+int traverseChildren(ClassRegister *cReg, const char *parent, const char *chldn) {
+      char *child;
+      int rc = CMPI_RC_ERR_FAILED;
+      UtilList *ul = getChildren(cReg, parent);
+      
+      cReg->ft->rLock(cReg);
+   
+      if (ul) for (child=(char*)ul->ft->getFirst(ul); child; 
+                   child=(char*)ul->ft->getNext(ul)) {
+         if (strcasecmp(child,chldn)==0 ) {
+            rc=CMPI_RC_OK;
+            break;
+         }
+         else {
+            rc = traverseChildren(cReg, child, chldn);
+            if(rc == CMPI_RC_OK) break;
+         }
+      }
+      
+      cReg->ft->rUnLock(cReg);
+      return rc;
+}
+   
 CMClassMIStub(ClassProvider, ClassProvider, _broker, CMNoHook);
 
 CMMethodMIStub(ClassProvider, ClassProvider, _broker, CMNoHook);
