@@ -66,6 +66,8 @@ Control init[] = {
    {"enableHttps",      2, "false"},
    {"httpsProcs",       1, "8"},
    {"httpLocalOnly",    2, "false"},
+   {"httpUserSFCB", 2, "true"},
+   {"httpUser", 0, ""},
 #ifdef HAVE_JDBC
    {"dbpPort",         1, "5980"},
    {"enableDbp",         2, "true"},
@@ -75,6 +77,7 @@ Control init[] = {
    {"dbpsProcs",        1, "8"},
 #endif
 #ifdef HAVE_SLP
+   {"enableSlp",         2, "true"},
    {"slpRefreshInterval",         1, "600"},
    {"slpHostnameLib",   0, "sfcSlpHostname"},
 #endif
@@ -92,6 +95,8 @@ Control init[] = {
    {"providerSampleInterval",  1, "30"},
    {"providerTimeoutInterval", 1, "60"},
    {"providerAutoGroup", 2, "true"},
+   {"providerDefaultUserSFCB", 2, "true"},
+   {"providerDefaultUser", 0, ""},
 
    {"sslKeyFilePath",   0, SFCB_CONFDIR "/file.pem"},
    {"sslCertificateFilePath", 0, SFCB_CONFDIR "/server.pem"},
@@ -193,7 +198,6 @@ int setupControl(char *fn)
          break;
       }
    }
-
    if (stmt) free(stmt);
 
 
