@@ -58,7 +58,9 @@
 
 //unsigned long exFlags = 0;
 static char *name;
-
+#if defined USE_SSL
+static SSL_CTX *ctx;
+#endif
 int sfcbSSLMode = 0;
 static int doFork = 0;
 static int doBa;
@@ -113,6 +115,18 @@ extern int semGetValue(int semid, int semnum);
 
 ResultSet * rs;
 
+#if defined USE_SSL
+void
+handleSSLerror(const char *file, int lineno, const char *msg)
+{
+  mlogf(M_ERROR, M_SHOW, "\n*** %s:%i %s -- exiting\n", file, lineno, msg);
+#ifdef SFCB_DEBUG
+  ERR_print_errors_fp(stderr);
+#endif
+  exit(-1);
+}
+#endif
+
 void initDbpProcCtl(int p)
 {
    dbpProcSemKey=ftok(SFCB_BINARY,'D');
@@ -522,7 +536,8 @@ static void handleSigUsr1(int sig)
  */
 int dbpDaemon(int argc, char *argv[], int sslMode, int sfcbPid) {//int argc, char *argv[], int sslMode) {
 	struct sockaddr_in sin;
-   int sz,sin_len,ru;
+   int sin_len,ru;
+   socklen_t sz;
    char *cp;
    long procs, port;
    int listenFd, connFd;
