@@ -1,6 +1,6 @@
 
 /*
- * $Id: fileRepository.c,v 1.18 2009/01/09 21:08:29 mchasal Exp $
+ * $Id: fileRepository.c,v 1.20 2009/11/09 21:03:30 buccella Exp $
  *
  * © Copyright IBM Corp. 2005, 2007
  *
@@ -90,7 +90,7 @@ static int getIndexRecordCase(BlobIndex * bi, const char *key, size_t keyl, char
   int slen;
   
   /* check index range */
-  if (bi->next > bi->dSize) {
+  if (bi->next >= bi->dSize) {
     return -1;
   }
   tokenptr = bi->index + bi->next;
@@ -206,7 +206,7 @@ void freeBlobIndex(BlobIndex **bip, int all)
    if (bi->dir) { free(bi->dir); bi->dir=NULL; }
    if (bi->fnx) { free(bi->fnx); bi->fnx=NULL; }
    if (bi->fnd) { free(bi->fnd); bi->fnd=NULL; }
-   if (all) if (bi->index)  { free(bi->index); bi->fnd=NULL; }
+   if (all) if (bi->index)  { free(bi->index); bi->index=NULL; }
    bi->freed=-1;
    if (bi->fd) fclose(bi->fd);
    if (bi->fx) fclose(bi->fx);
@@ -400,6 +400,7 @@ int getIndex(const char *ns, const char *cls, int elen, int mki, BlobIndex **bip
       bi->index=malloc(bi->aSize);
       fseek(bi->fx,0,SEEK_SET);
       fread(bi->index,bi->dSize,1,bi->fx);
+      bi->index[bi->dSize]=0;
     }
     *bip=bi;
     return 1;
