@@ -9,8 +9,8 @@ CIMSCHEMA_SOURCE=${CIMSCHEMA_SOURCE:-}
 CIMSCHEMA_MOF=${CIMSCHEMA_MOF:-}
 CIMSCHEMA_SUBDIRS=${CIMSCHEMA_SUBDIRS:-}
 
-MOFZIPURL=${CIMSCHEMA_SOURCE:-"http://www.dmtf.org/standards/cim/cim_schema_v217/cimv217Final-MOFs.zip"}
-SCHEMAMOF=${CIMSCHEMA_MOF:-cimv217.mof}
+MOFZIPURL=${CIMSCHEMA_SOURCE:-"http://www.dmtf.org/standards/cim/cim_schema_v2250/cim_schema_2.25.0Final-MOFs.zip"}
+SCHEMAMOF=${CIMSCHEMA_MOF:-cim_schema_2.25.0.mof}
 
 # zip file for schema v2.17 (and others) contain subdirectories.  v2.9 does not
 HASSUBDIRS=yes
