@@ -1,5 +1,5 @@
 /*
- * $Id: instance.c,v 1.45 2008/11/21 20:23:51 mchasal Exp $
+ * $Id: instance.c,v 1.51 2010/05/20 23:45:20 buccella Exp $
  *
  * © Copyright IBM Corp. 2005, 2007
  *
@@ -184,9 +184,8 @@ CMPIData __ift_internal_getPropertyAt(const CMPIInstance * ci, CMPICount i,
       rv.value.string = 
 	 sfcb_native_new_CMPIString(rv.value.string->hdl, NULL,0);
    }  else if (rv.type == CMPI_ref) {
-      char *msg;
       rv.value.ref = getObjectPath(
-         (char*)ClObjectGetClString(&inst->hdr, (ClString*)&rv.value.chars), &msg);
+         (char*)ClObjectGetClString(&inst->hdr, (ClString*)&rv.value.chars), NULL);
    }
    else if (rv.type & CMPI_ARRAY && rv.value.array) {
       rv.value.array =
@@ -279,6 +278,22 @@ static CMPIStatus __ift_setProperty(const CMPIInstance * instance,
    CMReturn(CMPI_RC_OK);
 }
 
+CMPIStatus filterFlagProperty(CMPIInstance* ci, const char* id) {
+
+   CMPIStatus     st = { CMPI_RC_OK, 0 };
+   ClInstance     *inst = (ClInstance *) ci->hdl;
+   ClSection      *prps = &inst->properties;
+   int             i;
+
+   if ((i = ClObjectLocateProperty(&inst->hdr, prps, id)) != 0) {
+      ClInstanceFilterFlagProperty(inst, i-1);
+   }
+   else
+      st.rc = CMPI_RC_ERR_NOT_FOUND;
+
+   return st;
+}
+
 static CMPIStatus __ift_setObjectPath(CMPIInstance * inst,
                                       const CMPIObjectPath * cop)
 {
@@ -304,6 +319,7 @@ static CMPIStatus __ift_setObjectPath(CMPIInstance * inst,
       ns = "*NoNameSpace*";
       cn = "*NoClassName*";
       tmp1.rc=tmp2.rc=tmp3.rc=CMPI_RC_OK;
+      tmp1.msg=tmp2.msg=tmp3.msg=NULL;
    }
 
    if (tmp1.rc != CMPI_RC_OK || tmp2.rc != CMPI_RC_OK || tmp3.rc != CMPI_RC_OK) {
@@ -913,6 +929,13 @@ static void instFillDefaultProperties(struct native_instance *inst,
       while (pc > 0) {
 	 pc -= 1;
 	 pd = cc->ft->getPropertyAt(cc,pc,&pn,&ps);
+
+	 /* if this prop is an EmbeddedObject, force type to CMPI_instance to allow CMSetProperty with a CMPI_Instance */
+	 CMPIData pqd = cc->ft->getPropQualifier(cc, CMGetCharsPtr(pn, NULL), "EmbeddedObject", NULL);
+	 if ((pqd.state == CMPI_goodValue) && (pqd.value.boolean == 1)) {
+	    pd.type = CMPI_instance;
+	 }
+
 	 if (ps.rc == CMPI_RC_OK && pn ) {
 	    vp = &pd.value;
 	    if (pd.state & CMPI_nullValue) {
