@@ -38,8 +38,6 @@
 #include "native.h"
 #include "objectpath.h"
 
-#include "cimslp.h"
-
 #define LOCALCLASSNAME "InternalProvider"
 
 static char * interopNs = "root/interop";
@@ -90,7 +88,7 @@ static CMPIInstance *instifyBlob(void * blob) {
 	}	
 }
 
-static CMPIInstance* ipGetBlob(const char *ns, const char *cls, char *id, int *len) {
+static CMPIInstance* ipGetBlob(const char *ns, const char *cls, const char *id, int *len) {
 	void *blob=getBlob(ns, cls, id, len);
    	return instifyBlob(blob);
 }
@@ -160,19 +158,6 @@ static BlobIndex *_getIndex(const char *ns, const char *cn)
  }
 
 
-extern int isChild(const char *ns, const char *parent, const char* child);
-
-static int isa(const char *sns, const char *child, const char *parent)
-{
-   int rv;
-   _SFCB_ENTER(TRACE_INTERNALPROVIDER, "isa");
-   
-   if (strcasecmp(child,parent)==0) return 1;
-   rv=isChild(sns,parent,child);
-   _SFCB_RETURN(rv);
-}
-
-
 /* ------------------------------------------------------------------ *
  * Instance MI Cleanup
  * ------------------------------------------------------------------ */
@@ -210,7 +195,6 @@ CMPIStatus InternalProviderEnumInstanceNames(CMPIInstanceMI * mi,
    int i,ac=0;
    char copKey[8192]="";
    char *kp;
-   char *msg;
    CMPIArgs *in,*out;
    CMPIObjectPath *op;
    CMPIArray *ar;
@@ -237,7 +221,7 @@ CMPIStatus InternalProviderEnumInstanceNames(CMPIInstanceMI * mi,
             strcat(copKey,".");
             strncat(copKey,kp,ekl);
 	    
-            cop=getObjectPath(copKey,&msg);
+            cop=getObjectPath(copKey,NULL);
             if (cop) CMReturnObjectPath(rslt, cop);
             else {
 	      CMPIStatus st = { CMPI_RC_ERR_FAILED, NULL };
@@ -362,7 +346,7 @@ CMPIInstance *internalProviderGetInstance(const CMPIObjectPath * cop, CMPIStatus
    int len;
    CMPIString *cn = CMGetClassName(cop, NULL);
    CMPIString *ns = CMGetNameSpace(cop, NULL);
-   char *key = normalizeObjectPathChars(cop);
+   char *key = normalizeObjectPathCharsDup(cop);
    CMPIInstance *ci=NULL;
    const char *nss=ns->ft->getCharPtr(ns,NULL);
    const char *cns=cn->ft->getCharPtr(cn,NULL);
@@ -374,6 +358,7 @@ CMPIInstance *internalProviderGetInstance(const CMPIObjectPath * cop, CMPIStatus
    
    if (testNameSpace(bnss,rc)==0) {
       _SFCB_TRACE(1,("--- Invalid namespace %s",nss));
+      free(key);
       _SFCB_RETURN(NULL);
    }
 
@@ -385,6 +370,7 @@ CMPIInstance *internalProviderGetInstance(const CMPIObjectPath * cop, CMPIStatus
    }   
 
    *rc=st;
+   free(key);
    _SFCB_RETURN(ci);
 }
 
@@ -427,7 +413,7 @@ CMPIStatus InternalProviderCreateInstance(CMPIInstanceMI * mi,
    void *blob;
    CMPIString *cn = CMGetClassName(cop, NULL);
    CMPIString *ns = CMGetNameSpace(cop, NULL);
-   char *key = normalizeObjectPathChars(cop);
+   char *key = normalizeObjectPathCharsDup(cop);
    const char *nss=ns->ft->getCharPtr(ns,NULL);
    const char *cns=cn->ft->getCharPtr(cn,NULL);
    const char *bnss=repositoryNs(nss);
@@ -435,12 +421,22 @@ CMPIStatus InternalProviderCreateInstance(CMPIInstanceMI * mi,
    _SFCB_ENTER(TRACE_INTERNALPROVIDER, "InternalProviderCreateInstance");
    
    if (testNameSpace(bnss,&st)==0) {
-      return st;
+	   free(key);
+	   _SFCB_RETURN(st);
+   }
+
+   CMPIConstClass *cc =getConstClass(nss, cns);
+   /* per DSP0004 2.5.2 */
+   if (cc!=NULL && cc->ft->isAbstract(cc) != 0) {
+      CMPIStatus st = { CMPI_RC_ERR_NOT_SUPPORTED, NULL };
+      free(key);
+      _SFCB_RETURN(st);
    }
 
    if (existingBlob(bnss,cns,key)) {
       CMPIStatus st = { CMPI_RC_ERR_ALREADY_EXISTS, NULL };
-      return st;
+      free(key);
+      _SFCB_RETURN(st);
    }
 
    len=getInstanceSerializedSize(ci);
@@ -451,26 +447,15 @@ CMPIStatus InternalProviderCreateInstance(CMPIInstanceMI * mi,
       CMPIStatus st = { CMPI_RC_ERR_FAILED, NULL };
       st.msg=sfcb_native_new_CMPIString("Unable to write to repository",NULL,0);
       free(blob);
-      return st;
+      free(key);
+      _SFCB_RETURN(st);
    }
    free(blob);
    
    if (rslt) { 
      CMReturnObjectPath(rslt, cop);
-     if (isa(nss, cns, "cim_registeredprofile")) {
-        CMPIArray* atArray;
-        atArray = CMGetProperty(ci, "AdvertiseTypes", &st).value.array;
-#ifdef HAVE_SLP
-        if (st.rc == CMPI_RC_OK || 
-            atArray != NULL || 
-            CMGetArrayElementAt(atArray, 0, &st).value.uint16 == 3) {
-	  if (slppid > 1) /* sanity check */
-            kill(slppid, SIGHUP);  /* restart SLP to update RegisteredProfiles */
-        }
-#endif
-     }
    }
-
+   free(key);
    _SFCB_RETURN(st);
 }
 
@@ -486,7 +471,7 @@ CMPIStatus InternalProviderModifyInstance(CMPIInstanceMI * mi,
    void *blob;
    CMPIString *cn = CMGetClassName(cop, NULL);
    CMPIString *ns = CMGetNameSpace(cop, NULL);
-   char *key = normalizeObjectPathChars(cop);
+   char *key = normalizeObjectPathCharsDup(cop);
    const char *nss=ns->ft->getCharPtr(ns,NULL);
    const char *cns=cn->ft->getCharPtr(cn,NULL);
    const char *bnss=repositoryNs(nss);
@@ -495,12 +480,14 @@ CMPIStatus InternalProviderModifyInstance(CMPIInstanceMI * mi,
    _SFCB_ENTER(TRACE_INTERNALPROVIDER, "InternalProviderSetInstance");
    
    if (testNameSpace(bnss,&st)==0) {
-      return st;
+	  free(key);
+      _SFCB_RETURN(st);
    }
 
    if (existingBlob(bnss,cns,key)==0) {
       CMPIStatus st = { CMPI_RC_ERR_NOT_FOUND, NULL };
-      return st;
+      free(key);
+      _SFCB_RETURN(st);
    }
    
    if(properties) {
@@ -516,18 +503,7 @@ CMPIStatus InternalProviderModifyInstance(CMPIInstanceMI * mi,
    getSerializedInstance(ci,blob);
    addBlob(bnss,cns,key,blob,(int)len);
    free(blob);
-
-   if (isa(nss, cns, "cim_registeredprofile")) {
-      CMPIArray* atArray;
-      atArray = CMGetProperty(ci, "AdvertiseTypes", &st).value.array;
-      if (st.rc == CMPI_RC_OK || 
-          atArray != NULL || 
-          CMGetArrayElementAt(atArray, 0, &st).value.uint16 == 3) {
-
-        kill(slppid, SIGHUP);  /* restart SLP to update RegisteredProfiles */
-      }
-   }
-
+   free(key);
    _SFCB_RETURN(st);
 }
 
@@ -539,7 +515,7 @@ CMPIStatus InternalProviderDeleteInstance(CMPIInstanceMI * mi,
    CMPIStatus st = { CMPI_RC_OK, NULL };
    CMPIString *cn = CMGetClassName(cop, NULL);
    CMPIString *ns = CMGetNameSpace(cop, NULL);
-   char *key = normalizeObjectPathChars(cop);
+   char *key = normalizeObjectPathCharsDup(cop);
    const char *nss=ns->ft->getCharPtr(ns,NULL);
    const char *cns=cn->ft->getCharPtr(cn,NULL);
    const char *bnss=repositoryNs(nss);
@@ -547,20 +523,18 @@ CMPIStatus InternalProviderDeleteInstance(CMPIInstanceMI * mi,
    _SFCB_ENTER(TRACE_INTERNALPROVIDER, "InternalProviderDeleteInstance");
    
    if (testNameSpace(bnss,&st)==0) {
-      return st;
+	   free(key);
+	   _SFCB_RETURN(st);
    }
 
    if (existingBlob(bnss,cns,key)==0) {
       CMPIStatus st = { CMPI_RC_ERR_NOT_FOUND, NULL };
-      return st;
+      free(key);
+      _SFCB_RETURN(st);
    }
 
    deleteBlob(bnss,cns,key);
-
-   if (isa(nss, cns, "cim_registeredprofile")) {
-          kill(slppid, SIGHUP); /* restart SLP to update RegisteredProfiles */
-   }
-
+   free(key);
    _SFCB_RETURN(st);
 }
 
