@@ -36,6 +36,7 @@
 #include "native.h"
 #include "objectpath.h"
 #include <time.h>
+#include "instance.h"
 
 #define LOCALCLASSNAME "InteropProvider"
 
@@ -49,6 +50,7 @@ extern CMPIObjectPath *getObjectPath(char *path, char **msg);
 extern void closeProviderContext(BinRequestContext* ctx);
 extern void setStatus(CMPIStatus *st, CMPIrc rc, char *msg);
 extern int testNameSpace(char *ns, CMPIStatus *st);
+extern void memLinkObjectPath(CMPIObjectPath *op);
 
 /* ------------------------------------------------------------------------- */
 
@@ -526,8 +528,8 @@ static CMPIStatus processSubscription(
 	const CMPIObjectPath *cop)
 {
    CMPIStatus st = { CMPI_RC_OK, NULL };
-   Handler *ha;
-   Filter *fi;
+   Handler *ha = NULL;
+   Filter *fi = NULL;
    Subscription *su;
    CMPIObjectPath *op;
    char *key,*skey;
@@ -547,9 +549,13 @@ static CMPIStatus processSubscription(
       
    _SFCB_TRACE(1,("--- getting new subscription filter"));
    op = CMGetProperty(ci, "filter", &st).value.ref;
+   if (op) {
    key = normalizeObjectPathCharsDup(op);
+     if (key) {
    fi = getFilter(key);
-   if(key) free(key);
+       free(key);
+     }
+   }
    
    if (fi == NULL) {
       _SFCB_TRACE(1,("--- cannot find specified subscription filter"));
@@ -559,9 +565,13 @@ static CMPIStatus processSubscription(
    
    _SFCB_TRACE(1,("--- getting new subscription handle"));
    op = CMGetProperty(ci, "handler", &st).value.ref;
+   if (op) {
    key = normalizeObjectPathCharsDup(op);
+     if (key) {
    ha = getHandler(key);
-   if(key) free(key);
+       free(key);
+     }
+   }
       
    if (ha == NULL) {
       _SFCB_TRACE(1,("--- cannot find specified subscription handler"));
@@ -667,6 +677,26 @@ void initInterOp(
    _SFCB_EXIT(); 
 }
 
+/*
+ for CIM_IndicationSubscription we use the DeliveryFailureTime property to track
+ when indication delivery first failed.  However, this property is not a part of
+ the mof supplied by the DMTF, so we need to filter it out of instances being
+ returned to the client
+*/
+void
+filterInternalProps(CMPIInstance* ci)
+{
+
+  CMPIStatus      pst = { CMPI_RC_OK, NULL };
+  CMGetProperty(ci, "DeliveryFailureTime", &pst);
+  /* prop is set, need to clear it out */
+  if (pst.rc != CMPI_RC_ERR_NOT_FOUND) {
+    filterFlagProperty(ci, "DeliveryFailureTime");
+  }
+
+  return;
+}
+
 /* --------------------------------------------------------------------------*/
 /*                       Instance Provider Interface                         */
 /* --------------------------------------------------------------------------*/
@@ -724,9 +754,18 @@ CMPIStatus InteropProviderEnumInstances(
    enm = _broker->bft->enumerateInstances(_broker, ctxLocal, ref, properties, &st);
    CMRelease(ctxLocal);
                                       
-   while(enm && enm->ft->hasNext(enm, &st)) {
-       CMReturnInstance(rslt, (enm->ft->getNext(enm, &st)).value.inst);   
-   }   
+   while (enm && enm->ft->hasNext(enm, &st)) {
+
+     CMPIInstance* ci = (enm->ft->getNext(enm, &st)).value.inst;
+
+     /* need to check IndicationSubscription, since it may contain props used internally by sfcb */
+     CMPIObjectPath* cop = CMGetObjectPath(ci, &st);
+     if (strcasecmp(CMGetCharPtr(CMGetClassName(cop, NULL)), "cim_indicationsubscription") == 0) {
+       filterInternalProps(ci);
+     }
+
+     CMReturnInstance(rslt, ci);
+   }
    if(enm) CMRelease(enm);
    _SFCB_RETURN(st);
 }
@@ -750,6 +789,11 @@ CMPIStatus InteropProviderGetInstance(
 
    ci = _broker->bft->getInstance(_broker, ctxLocal, cop, properties, &st);
    if (st.rc==CMPI_RC_OK) {
+     /* need to check IndicationSubscription, since it may contain props used internally by sfcb */
+     if (strcasecmp(CMGetCharPtr(CMGetClassName(cop, NULL)), "cim_indicationsubscription") == 0) {
+       filterInternalProps(ci);
+     }
+
       CMReturnInstance(rslt, ci);
    }
    
@@ -774,6 +818,7 @@ CMPIStatus InteropProviderCreateInstance(
    const char *nss = ns->ft->getCharPtr(ns,NULL);
    CMPIContext *ctxLocal;
    CMPIInstance *ciLocal;
+   CMPIObjectPath *copLocal;
    CMPIValue valSNS;
 
    _SFCB_ENTER(TRACE_INDPROVIDER, "InteropProviderCreateInstance");
@@ -782,12 +827,13 @@ CMPIStatus InteropProviderCreateInstance(
    
    ciLocal = ci->ft->clone(ci, NULL);
    memLinkInstance(ciLocal);
+   copLocal = cop->ft->clone(cop, NULL);
+   memLinkObjectPath(copLocal);
    
    if(isa(nss, cns, "cim_indicationsubscription")) {
-   
       _SFCB_TRACE(1,("--- create cim_indicationsubscription"));
-      
-      st=processSubscription(_broker,ctx,ciLocal,cop);
+
+      st=processSubscription(_broker,ctx,ciLocal,copLocal);
    }
    else if (isa(nss, cns, "cim_indicationfilter")) {
       QLStatement *qs=NULL;
@@ -795,25 +841,54 @@ CMPIStatus InteropProviderCreateInstance(
       char *key=NULL,*ql,lng[16];
       CMPIString *lang=ciLocal->ft->getProperty(ciLocal,"querylanguage",&st).value.string;
       CMPIString *query=ciLocal->ft->getProperty(ciLocal,"query",&st).value.string;
-      CMPIString *sns=ciLocal->ft->getProperty(ciLocal,"SourceNamespace",&st).value.string;
+      CMPIBoolean iss=ciLocal->ft->getProperty(ciLocal,"individualsubscriptionsupported",&st).value.boolean;
+      CMPIValue iss_default = { .boolean=1 };
       
       _SFCB_TRACE(1,("--- create cim_indicationfilter"));
    
+      /* This property new as of 2.22.  Really only useful for 
+         FilterCollections. If FCs are implemented, this will need updating. */
+      if (st.rc == CMPI_RC_OK && iss == 0) {
+        setStatus(&st,CMPI_RC_ERR_NOT_SUPPORTED,"IndividualSubscriptionSupported property must be TRUE (FilterCollections not available)");
+        _SFCB_RETURN(st);         
+      }
+      else {
+        CMSetProperty(ciLocal, "IndividualSubscriptionSupported", &iss_default, CMPI_boolean);
+      }
+
       if (lang == NULL || lang->hdl == NULL
           || query == NULL ||  query->hdl == NULL) {
          setStatus(&st,CMPI_RC_ERR_FAILED,"Query and/or Language property not found");
          _SFCB_RETURN(st);         
       }
       
-      if(sns == NULL || sns->hdl == NULL) {
-      	  /* if sourcenamespace is NULL, the namespace of the filter
+      CMPIString *sns=ciLocal->ft->getProperty(ciLocal,"SourceNamespace",&st).value.string;
+      CMPIArray *snsa=ciLocal->ft->getProperty(ciLocal,"SourceNamespaces",&st).value.array;
+      if((sns == NULL || sns->hdl == NULL) && (snsa == NULL || snsa->hdl == NULL)) {
+      	  /* if sourcenamespace and sourcenamespaces are both  NULL, the namespace of the filter
       	   * registration is assumed */
-	sns = sfcb_native_new_CMPIString("root/interop", NULL,0);
+          sns = sfcb_native_new_CMPIString("root/interop", NULL,0);
           valSNS.string = sns;
-          ciLocal->ft->setProperty(ciLocal, "SourceNamespace", &valSNS,
-                                   CMPI_string);
+          ciLocal->ft->setProperty(ciLocal, "SourceNamespace", &valSNS, CMPI_string);
+          CMSetStatus(&st, CMPI_RC_OK);
+      } else if (sns == NULL || sns->hdl == NULL) {
+        // Sourcenamespaces is set, but sourcenamespace is not, put the 1st entry of
+        // SourceNamespaces in Sourcenamespace
+          sns = CMGetArrayElementAt(snsa, 0, NULL).value.string;
+          valSNS.string = sns;
+          ciLocal->ft->setProperty(ciLocal, "SourceNamespace", &valSNS, CMPI_string);
+          CMSetStatus(&st, CMPI_RC_OK);
       }
-      
+      // If SourceNamespaces isn't set, use the SourceNamespace
+      if(snsa == NULL || snsa->hdl == NULL) {
+          CMPIArray *snsa = internal_new_CMPIArray(MEM_TRACKED, 1 , CMPI_string, &st);
+          valSNS.string = sns;
+          CMSetArrayElementAt(snsa,0,&valSNS,CMPI_string);
+          valSNS.array = snsa;
+          ciLocal->ft->setProperty(ciLocal, "SourceNamespaces", &valSNS, CMPI_stringA);
+          CMSetStatus(&st, CMPI_RC_OK);
+       }
+        
       for (ql=(char*)lang->hdl,i=0,n=0,m=strlen(ql); i<m; i++) {
          if (ql[i]>' ') lng[n++]=ql[i];
          if (n>=15) break;
@@ -851,7 +926,7 @@ CMPIStatus InteropProviderCreateInstance(
     
    if (st.rc==CMPI_RC_OK) {
       ctxLocal = prepareUpcall((CMPIContext *)ctx);
-      CMReturnObjectPath(rslt, _broker->bft->createInstance(_broker, ctxLocal, cop, ciLocal, &st));
+      CMReturnObjectPath(rslt, _broker->bft->createInstance(_broker, ctxLocal, copLocal, ciLocal, &st));
       CMRelease(ctxLocal);
    }
     
@@ -1048,7 +1123,7 @@ CMPIStatus InteropProviderInvokeMethod(
             CMPIString *str=CDToString(_broker,su->ha->hop,NULL);
             CMPIString *ns=CMGetNameSpace(su->ha->hop,NULL);
             _SFCB_TRACE(1,("--- invoke handler %s %s",(char*)ns->hdl,(char*)str->hdl));
-            
+            CMAddArg(hin,"subscription",&su->sci,CMPI_instance);
             CBInvokeMethod(_broker,ctx,su->ha->hop,"_deliver",hin,NULL,&st);
             _SFCB_TRACE(1,("--- invoke handler status: %d",st.rc));
          }     
