@@ -1,5 +1,5 @@
 /*
- * $Id: mlog.h,v 1.4 2009/06/10 19:40:43 mchasal Exp $
+ * $Id: mlog.h,v 1.6 2010/04/07 20:54:11 buccella Exp $
  *
  * (C) Copyright IBM Corp. 2003, 2004
  *
@@ -27,7 +27,10 @@
 #define M_SHOW  1
 #define M_QUIET 0
 
-void startLogging(const char *name, int level);
+#define LOG_MSG_MAX 4096
+
+void startLogging(int level);
+void closeLogging();
 void mlogf(int priority, int errout, const char* fmt, ...);
 
 #endif
