@@ -40,6 +40,7 @@ ComSockets resultSockets;
 ComSockets sfcbSockets;
 ComSockets providerSockets;
 int localMode=1;
+int disableDefaultProvider=0;
 
 ComSockets *sPairs;
 int ptBase,htBase,stBase,htMax,stMax;
@@ -306,8 +307,11 @@ static int spRcvMsg(int *s, int *from, void **data, unsigned long *length, MqgSt
       }
       partRecvd = totalRecvd = 0;
       do {
-         if ((partRecvd = spGetMsg(s, NULL, *data + totalRecvd, *length - totalRecvd, mqg)) == -1)
+	if ((partRecvd = spGetMsg(s, NULL, *data + totalRecvd, *length - totalRecvd, mqg)) == -1) {
+            free(*data);
+            *data = 0;
             return spHandleError(s, em);
+	}
          totalRecvd += partRecvd;
          if (mqg->teintr) mqg->eintr=1;    
       } while (mqg->teintr) ;       
@@ -324,8 +328,11 @@ static int spRcvMsg(int *s, int *from, void **data, unsigned long *length, MqgSt
       *length = 256;
       partRecvd = totalRecvd = 0;
       do {
-         if ((partRecvd = spGetMsg(s, NULL, *data + totalRecvd, *length - totalRecvd, mqg)) == -1)
+	if ((partRecvd = spGetMsg(s, NULL, *data + totalRecvd, *length - totalRecvd, mqg)) == -1) {
+            free(*data);
+            *data = 0;
             return spHandleError(s, em);
+	}
          totalRecvd += partRecvd;
          if (mqg->teintr) mqg->eintr=1;    
       } while (mqg->teintr) ;       
@@ -333,12 +340,15 @@ static int spRcvMsg(int *s, int *from, void **data, unsigned long *length, MqgSt
 
    switch (spMsg.xtra) {
    case MSG_X_PROVIDER:
+   case MSG_X_SFCB_PROVIDER:
       *length = spMsg.segments;
       *data = spMsg.provId;
    case MSG_X_INVALID_NAMESPACE:
    case MSG_X_PROVIDER_NOT_FOUND:
    case MSG_X_INVALID_CLASS:
       _SFCB_RETURN(spMsg.xtra);
+   case MSG_X_NOT_SUPPORTED:
+      _SFCB_RETURN(spMsg.xtra);
    case MSG_X_LOCAL:
       *length = 0;
       *data = NULL;
