@@ -301,6 +301,38 @@ char *sfcb_pathToChars(const CMPIObjectPath * cop, CMPIStatus * rc, char *str)
    return str;
 }
 
+CMPIData
+__oft_getPropertyQualifier(const CMPIObjectPath * op,
+                           const char *pName,
+                           const char *qName,
+                           CMPIStatus *rc)
+{
+  CMPIConstClass *cl = NULL;
+  CMPIString     *cn = NULL;
+  CMPIString     *ns = NULL;
+  char           *cn_char = NULL;
+  char           *ns_char = NULL;
+  CMPIStatus      st = { 0, NULL };
+  CMPIData        retVal = { CMPI_null, CMPI_nullValue, {0} };
+  
+  cn = CMGetClassName(op, &st);
+  if(st.rc) goto invalid_handle;
+  cn_char = CMGetCharPtr(cn);
+ 
+  ns = CMGetNameSpace(op, &st);
+  if(st.rc) goto invalid_handle;
+  ns_char = CMGetCharPtr(ns);
+
+  cl = getConstClass(ns_char, cn_char);
+  if(cl) {
+    return(cl->ft->getPropQualifier(cl, pName, qName, rc));
+  }
+
+invalid_handle:
+  if(rc) rc->rc = CMPI_RC_ERR_INVALID_HANDLE;
+  return retVal;
+}
+
 static CMPIString *__oft_toString(const CMPIObjectPath * cop, CMPIStatus * rc)
 {
    char str[4096] = { 0 };
@@ -358,7 +390,7 @@ static CMPIObjectPathFT oft = {
    __oft_setNameSpaceFromObjectPath,
    __oft_setHostAndNameSpaceFromObjectPath,
    NULL,
-   NULL,
+   __oft_getPropertyQualifier,
    NULL,
    NULL,
     __oft_toString
@@ -513,15 +545,22 @@ static void addKey(CMPIObjectPath * op, char *kd, int ref)
    *val = 0;
    val++;
    if (ref) {
-     char * msg;
-     CMPIObjectPath *keyOp = getObjectPath(val,&msg);
+     CMPIObjectPath *keyOp = getObjectPath(val,NULL);
      op->ft->addKey(op, kd, (CMPIValue*)&keyOp, CMPI_ref);
    } else if (*val == '"') {
       val++;
       val[strlen(val) - 1] = 0;
       op->ft->addKey(op, kd, (CMPIValue *) val, CMPI_chars);
    }
+   else if (val[0] == '-' || val[0] == '+') {
+     CMPISint64 vali = atol(val);
+     CMPIValue v = (CMPIValue)vali;
+     op->ft->addKey(op, kd, &v, CMPI_sint64);
+   }
    else if (isdigit(*val)) {
+     CMPIUint64 vali = atol(val);
+     CMPIValue v = (CMPIValue)vali;
+     op->ft->addKey(op, kd, &v, CMPI_uint64);
    }
    else {
    }
@@ -539,7 +578,7 @@ CMPIObjectPath *getObjectPath(char *path, char **msg)
    }
    u = origu = strdup(path);
    last = u + strlen(u);
-   *msg = NULL;
+   if (msg) *msg = NULL;
 
    p = strchr(u, '.');
    if (!p) {
@@ -589,13 +628,13 @@ CMPIObjectPath *getObjectPath(char *path, char **msg)
          break;
       if (*p == '"') {
          if (*(p - 1) != '=') {
-            *msg = "Incorrectly quoted string 1";
+            if(msg) *msg = "Incorrectly quoted string 1";
             free(origu);
             return NULL;
          }
          p++;
          if ((p = strchr(p, '"')) == NULL) {
-            *msg = "Unbalanced quoted string";
+            if(msg) *msg = "Unbalanced quoted string";
             free(origu);
             return NULL;
          }
