@@ -1,6 +1,6 @@
 
 /*
- * $Id: providerDrv.c,v 1.77 2009/05/01 22:17:21 mchasal Exp $
+ * $Id: providerDrv.c,v 1.85 2010/04/14 20:06:24 buccella Exp $
  *
  * © Copyright IBM Corp. 2005, 2007
  *
@@ -264,18 +264,21 @@ int stopNextProc()
 
 static void stopProc(void *p)
 {
-   ProviderInfo *pInfo;
+   ProviderInfo *pInfo, *temp;
    CMPIContext *ctx = NULL;
 
    ctx = native_new_CMPIContext(MEM_NOT_TRACKED,NULL);
 //   for (pInfo=curProvProc->firstProv; pInfo; pInfo=pInfo->next) {
    for (pInfo=activProvs; pInfo; pInfo=pInfo->next) {
+   for (temp=activProvs;temp;temp=temp->next) {
+      if((strcmp(temp->providerName,pInfo->providerName)==0) && (strcmp(temp->className,pInfo->className)!=0)) break;
       if (pInfo->classMI) pInfo->classMI->ft->cleanup(pInfo->classMI, ctx);
       if (pInfo->instanceMI) pInfo->instanceMI->ft->cleanup(pInfo->instanceMI, ctx, 1);
       if (pInfo->associationMI) pInfo->associationMI->ft->cleanup(pInfo->associationMI, ctx, 1);
       if (pInfo->methodMI) pInfo->methodMI->ft->cleanup(pInfo->methodMI, ctx, 1);
       if (pInfo->indicationMI) pInfo->indicationMI->ft->cleanup(pInfo->indicationMI, ctx, 1);
       //dlclose(pInfo->library);
+    }
    }
    mlogf(M_INFO,M_SHOW,"---  stopped %s %d\n",processName,getpid());
    ctx->ft->release(ctx);
@@ -352,7 +355,7 @@ void* providerIdleThread()
    struct timespec idleTime;
    time_t next;
    int rc,val,doNotExit,noBreak=1;
-   ProviderInfo *pInfo;
+   ProviderInfo *pInfo, *temp;
    ProviderProcess *proc;
    CMPIContext *ctx = NULL;
    CMPIStatus crc;
@@ -385,6 +388,8 @@ void* providerIdleThread()
                      ctx = native_new_CMPIContext(MEM_TRACKED,NULL);
                      noBreak=0;
                      for (crc.rc=0,pInfo = activProvs; pInfo; pInfo = pInfo->next) {
+    		     for(temp=activProvs;temp;temp=temp->next) {
+			if((strcmp(temp->providerName,pInfo->providerName)==0) && (strcmp(temp->className,pInfo->className)!=0)) break;
                         if (pInfo->library==NULL) continue;
                         if (pInfo->indicationMI!=NULL) continue;
                         if (crc.rc==0 && pInfo->instanceMI) 
@@ -407,7 +412,8 @@ void* providerIdleThread()
 			   pthread_mutex_destroy(&pInfo->initMtx);
                         }   
                         else doNotExit=1;
-                     }  
+                       }
+		     }  
                      if (doNotExit==0) {
                         dumpTiming(currentProc);
                         _SFCB_TRACE(1, ("--- Exiting %s-%d",processName,currentProc));
@@ -551,7 +557,7 @@ static CMPIStatus getQualifierDeclMI(ProviderInfo *info, CMPIQualifierDeclMI **m
 */
 static int getProcess(ProviderInfo * info, ProviderProcess ** proc)
 {
-   int i;
+   int i,rc;
    static int seq=0;
 
    _SFCB_ENTER(TRACE_PROVIDERDRV, "getProcess");
@@ -610,7 +616,18 @@ static int getProcess(ProviderInfo * info, ProviderProcess ** proc)
             setSignal(SIGUSR1, handleSigUsr1,0);
             
             setSignal(SIGSEGV, handleSigSegv,SA_ONESHOT);
-            
+
+            // If requested, change the uid of the provider
+            if (info->uid != -1 ) {
+                _SFCB_TRACE(1,("--- Changing uid of provider, %s, to %d(%s)",info->providerName,info->uid,info->user));
+                // Set the real and effective uids
+                rc=setreuid(info->uid,info->uid);
+                if (rc == -1) {
+                    mlogf(M_ERROR,M_SHOW,"--- Changing uid for %s failed.\n",info->providerName);
+                    _SFCB_RETURN(-1);
+                }
+            }
+
             curProvProc=(*proc);
             resultSockets=sPairs[(*proc)->id+ptBase];
 
@@ -690,6 +707,7 @@ int forkProvider(ProviderInfo * info, OperationHdr * req, char **msg)
       sreq.className = setCharsMsgSegment(info->className);
       sreq.libName = setCharsMsgSegment(info->location);
       sreq.provName = setCharsMsgSegment(info->providerName);
+      sreq.parameters = setCharsMsgSegment(info->parms);
       sreq.hdr.flags = info->type;
       sreq.unload = info->unload;
       sreq.hdr.provId = getProvIds(info).ids;
@@ -1385,6 +1403,7 @@ static BinResponseHdr *invokeMethod(BinRequestHdr * hdr, ProviderInfo * info,
    ctx->ft->addEntry(ctx,CMPIInvocationFlags,(CMPIValue*)&flgs,CMPI_uint32);
    ctx->ft->addEntry(ctx,CMPIPrincipal,(CMPIValue*)req->principal.data,CMPI_chars);
    ctx->ft->addEntry(ctx,"CMPISessionId",(CMPIValue*)&hdr->sessionId,CMPI_uint32);
+   if (info->parms) ctx->ft->addEntry(ctx,"sfcbProviderParameters",(CMPIValue*)info->parms,CMPI_chars);
 
    if (req->hdr.count>5) {
       int i,s,n;
@@ -2047,7 +2066,25 @@ static BinResponseHdr *activateFilter(BinRequestHdr *hdr, ProviderInfo* info,
       char *lang=(char*)req->language.data;
       type=(char*)req->type.data;
       char *sns=(char*)req->sns.data;
+
       se=(NativeSelectExp*)NewCMPISelectExp(query,lang,sns,NULL,&rci);
+      if (rci.rc != CMPI_RC_OK) {
+	mlogf(M_DEBUG, M_SHOW, "Failed to parse query (%s). Error code(%d).\n", query, rci.rc);
+	resp = errorResp(&rci);
+	_SFCB_RETURN(resp);
+      }
+      else if (se == NULL) {
+	/*
+	 * If se is NULL, rci.rc should have an error code set, but check for
+	 * the NULL case just in case something went wrong.
+	 */
+	mlogf(M_ERROR, M_SHOW, "Unknown error parsing query (%s).\n", query);
+	rci.rc = CMPI_RC_ERR_FAILED;
+	resp = errorResp(&rci);
+	_SFCB_RETURN(resp);
+      }
+
+
       makeActive=1;
       se->filterId=req->filterId;
       prev=se->next=activFilters;
@@ -2323,6 +2360,7 @@ int initProvider(ProviderInfo *info, unsigned int sessionId, char** errorStr)
 
    pthread_mutex_lock(&info->initMtx);
    if (info->initialized==0) {  
+     info -> initialized = 1;
    
      ctx->ft->addEntry(ctx,CMPIInvocationFlags,(CMPIValue*)&flgs,CMPI_uint32);
      ctx->ft->addEntry(ctx,CMPIPrincipal,(CMPIValue*)"$$",CMPI_chars);
@@ -2393,6 +2431,7 @@ int initProvider(ProviderInfo *info, unsigned int sessionId, char** errorStr)
      
      if (rc) {
        rc = -2;
+       info -> initialized = 0;
        if (errstr != NULL) {
           *errorStr = sfcb_snprintf("Error initializing provider %s from %s for class %s.  %s", 
                            info->providerName, info->location, 
@@ -2403,11 +2442,10 @@ int initProvider(ProviderInfo *info, unsigned int sessionId, char** errorStr)
                            info->providerName, info->location, info->className);
        }
      } else {
-       info -> initialized = 1;
        *errorStr = NULL;
      }
-     pthread_mutex_unlock(&info->initMtx);
    }
+   pthread_mutex_unlock(&info->initMtx);
    if (errstr != NULL) free(errstr); 
 
    _SFCB_RETURN(rc);
@@ -2482,6 +2520,7 @@ static BinResponseHdr *loadProvider(BinRequestHdr * hdr, ProviderInfo * info, in
    info->className = strdup((char*)req->className.data);
    info->location = strdup((char*)req->libName.data);
    info->providerName = strdup((char*)req->provName.data);
+   if (req->parameters.data) info->parms = strdup((char*)req->parameters.data);
    info->type = req->hdr.flags;
    info->unload = req->unload;
    info->providerSockets = providerSockets;
@@ -2647,7 +2686,7 @@ static void *processProviderInvocationRequestsThread(void *prms)
          doLoadProvider(pInfo,dlName, 512);
       }  
       
-      if (pInfo && pInfo->initialized == 0) {
+      if (pInfo) {
 	initRc=initProvider(pInfo,req->sessionId, &errstr);
 	_SFCB_TRACE(1, ("--- Provider initialization rc %d",initRc));
       }
@@ -2776,7 +2815,11 @@ void processProviderInvocationRequests(char *name)
                      (void **) &parms->req, &rl, &mqg);
       if (mqg.rdone) {
 	int debug_break = 0;
-         if (rc!=0)mlogf(M_ERROR,M_SHOW,"oops\n");               
+	if (rc!=0) {
+	  mlogf(M_ERROR,M_SHOW, "spRecvReq returned error %d. Skipping message.\n", rc);
+	  free(parms);
+	  continue;
+	}               
 
          _SFCB_TRACE(1, ("--- Got something op:%d-prov:%p on R%d-%lu", 
             parms->req->operation,parms->req->provId,
