@@ -1,6 +1,6 @@
 
 /*
- * $Id: providerMgr.c,v 1.61 2009/04/17 21:42:27 smswehla Exp $
+ * $Id: providerMgr.c,v 1.74 2010/05/17 20:24:08 buccella Exp $
  *
  * © Copyright IBM Corp. 2005, 2007
  *
@@ -38,6 +38,8 @@
 #include "selectexp.h"
 #include "config.h"
 
+#include <pthread.h>
+
 #ifdef HAVE_QUALREP
 #include "qualifier.h"
 #endif
@@ -61,6 +63,8 @@ int enumUnLock(CMPIEnumeration * ) ;
 #endif
 
 
+static pthread_mutex_t resultsocketMutex = PTHREAD_MUTEX_INITIALIZER;
+
 extern CMPIBroker *Broker;
 
 extern ProviderInfo *qualiProvInfoPtr;
@@ -237,8 +241,13 @@ static ProviderInfo *lookupProvider(long type, char *className, char *nameSpace,
       CMRelease(cc);
    }
     
-   _SFCB_TRACE(1,("Default provider for %s",className));
-   _SFCB_RETURN(defaultProvInfoPtr);
+   if(!disableDefaultProvider) {
+    _SFCB_TRACE(1,("Default provider for %s",className));
+    _SFCB_RETURN(defaultProvInfoPtr);
+   }
+   else {
+    _SFCB_RETURN(NULL);
+   }
 }
 
 
@@ -368,7 +377,7 @@ static void lookupProviderList(long type, int *requestor, OperationHdr * req)
    if (providers) {
       count = (n=providers->ft->size(providers)) - 1;
       _SFCB_TRACE(1,("--- found %d providers",count));
-
+     if(n) {
       for (info = providers->ft->getFirst(providers); info;
            info = providers->ft->getNext(providers), n--) {
          if (info->type!=FORCE_PROVIDER_NOTFOUND &&
@@ -388,6 +397,11 @@ static void lookupProviderList(long type, int *requestor, OperationHdr * req)
          }
       }
    }
+   else {
+      rc=MSG_X_NOT_SUPPORTED;
+      spSendCtlResult(requestor, &dmy, rc, 0, NULL, req->options);
+     }
+   }
    else {
       if (st.rc==CMPI_RC_ERR_INVALID_NAMESPACE) rc=MSG_X_INVALID_NAMESPACE;
       else rc=MSG_X_INVALID_CLASS;
@@ -481,31 +495,48 @@ static ProviderInfo *getAssocProvider(char *className, char *nameSpace)
    }
 
    info = (ProviderInfo *) assocProviderHt->ft->get(assocProviderHt, className);
-   if (info) _SFCB_RETURN(info);
+   /* there's a matching provider in the list, return it if the ns is right */
+   while(info) {
+      if (nameSpaceOk(info,nameSpace)) {
+         _SFCB_TRACE(1,("Provider found for %s",className));
+         _SFCB_RETURN(info);
+      }
+      info = info->nextInRegister;
+   }
 
    cls = className ? strdup(className) : NULL;
    while (cls != NULL) {
-      info = pReg->ft->getProvider(pReg, cls, type);
-      if (info) {
-	assocProviderHt->ft->put(assocProviderHt, strdup(className), info);
-	free(cls);
-	_SFCB_RETURN(info);
-      }
-      else {
-         CMPIConstClass *cc = (CMPIConstClass *) _getConstClass(nameSpace, cls, &rc);
-	 free(cls);
-         if (cc == NULL) {
-            _SFCB_RETURN(NULL);
-         }
-         cls = (char*)cc->ft->getCharSuperClassName(cc);
-         if (cls) {
-	   cls = strdup(cls);
-	 }
-	 CMRelease(cc);
-      }
+     /* find the FIRST matching prov for this class; add to ht even if ns doesn't match */
+     info = pReg->ft->getProvider(pReg, cls, type);
+     if (info && assocProviderHt->ft->get(assocProviderHt, cls) == NULL) {
+       assocProviderHt->ft->put(assocProviderHt, strdup(cls), info);
+     }
+     while (info) {
+       /* if ns matches, we can return; don't add to ht; first in list is already there */
+       if (nameSpaceOk(info, nameSpace)) {
+         free(cls);
+         _SFCB_RETURN(info);
+       }
+       info=info->nextInRegister;
+     }
+
+     CMPIConstClass *cc = (CMPIConstClass *) _getConstClass(nameSpace, cls, &rc);
+     free(cls);
+     if (cc == NULL) {
+       _SFCB_RETURN(NULL);
+     }
+     cls = (char*)cc->ft->getCharSuperClassName(cc);
+     if (cls) {
+       cls = strdup(cls);
+     }
+     CMRelease(cc);
    }
    
-   _SFCB_RETURN(defaultProvInfoPtr);
+   if(!disableDefaultProvider) {
+      _SFCB_RETURN(defaultProvInfoPtr);
+   }
+   else
+      _SFCB_RETURN(NULL);
 }
 
 static int addAssocProviders(char *className, char *nameSpace, UtilList * providerList)
@@ -537,10 +568,13 @@ static int addAssocProviders(char *className, char *nameSpace, UtilList * provid
    }
    else children = _getAssocClassNames(nameSpace);
 
-   if (children) for (child = children->ft->getFirst(children); child;
+   if (children) {
+     for (child = children->ft->getFirst(children); child;
            child = children->ft->getNext(children)) {
          rc = addAssocProviders(child, nameSpace, providerList);
          if (rc) _SFCB_RETURN(rc);
+     }
+     CMRelease(children);
    }
    _SFCB_RETURN(rc);
 }
@@ -613,7 +647,7 @@ static void assocProviderList(int *requestor, OperationHdr * req)
             count--, NULL, req->options);
       }
    }
-   else {
+   else if(disableDefaultProvider) {
       /* When there is no provider for an assocClass we do not want to produce
        * an error message. So we return the default provider and expect it
        * to produce a nice and empty result */
@@ -634,6 +668,10 @@ static void assocProviderList(int *requestor, OperationHdr * req)
 	 _SFCB_ABORT();
        }     
    }
+   else {
+       spSendCtlResult(requestor, &sfcbSockets.send, MSG_X_PROVIDER_NOT_FOUND,
+            count--, NULL, req->options);
+   }
    _SFCB_EXIT();
 }
 
@@ -691,7 +729,11 @@ static ProviderInfo *getMethodProvider(char *className, char *nameSpace)
    
    if (interopClass(className)) _SFCB_RETURN(forceNoProvInfoPtr);
    
-   _SFCB_RETURN(defaultProvInfoPtr);
+   if(!disableDefaultProvider) {
+	  _SFCB_RETURN(defaultProvInfoPtr);
+   }
+   else
+	  _SFCB_RETURN(NULL);
 }
 
 static void classProvider(int *requestor, OperationHdr * req)
@@ -745,6 +787,7 @@ static void methProvider(int *requestor, OperationHdr * req)
    char *className = (char *) req->className.data;
    char *nameSpace = (char *) req->nameSpace.data;
    ProviderInfo *info;
+   short          retcode;
    
    _SFCB_ENTER(TRACE_PROVIDERMGR, "methProvider");
    if (strcmp(className, "$ClassProvider$") == 0)
@@ -754,7 +797,15 @@ static void methProvider(int *requestor, OperationHdr * req)
       if (info->type!=FORCE_PROVIDER_NOTFOUND &&
           (rc = forkProvider(info, req, NULL)) == CMPI_RC_OK) {
          _SFCB_TRACE(1,("--- responding with  %s %p",info->providerName,info));
-         spSendCtlResult(requestor, &info->providerSockets.send, MSG_X_PROVIDER,
+       if(!(req->options & OH_Internal)
+          && info->location
+          && *info->location
+          && (strncmp(info->location, "sfc", 3) == 0)) {
+         retcode = MSG_X_SFCB_PROVIDER;
+       } else {
+         retcode = MSG_X_PROVIDER;
+       }
+         spSendCtlResult(requestor, &info->providerSockets.send, retcode,
             0, getProvIds(info).ids, req->options);
       }                   
       else {
@@ -886,10 +937,10 @@ void processProviderMgrRequests()
    if (interOpProvInfoPtr!=forceNoProvInfoPtr) {
      sleep(2);
      startUpProvider("root/interop","$InterOpProvider$");
-     sleep(2);
-     startUpProvider("root/interop","$ProfileProvider$");
    }   
 #endif
+
+   startUpProvider("root/interop","$ProfileProvider$");
       
    for (;;) {
       MgrHandler hdlr; 
@@ -991,7 +1042,10 @@ int getProviderContext(BinRequestContext * ctx, OperationHdr * ohdr)
    ((OperationHdr *) buf)->className.data = (void *) l;
    l += ohdr->className.length;
    
-   if (localMode) sockets=resultSockets;
+   if (localMode){
+	   pthread_mutex_lock(&resultsocketMutex);
+	   sockets=resultSockets;
+   }
    else sockets=getSocketPair("getProviderContext");   
    
    _SFCB_TRACE(1,("--- Sending mgr request - to %d from %d", sfcbSockets.send,
@@ -1002,9 +1056,12 @@ int getProviderContext(BinRequestContext * ctx, OperationHdr * ohdr)
    if (rc < 0) {
       mlogf(M_ERROR,M_SHOW,"--- spSendReq/spSendMsg failed to send on %d (%d)\n", sfcbSockets.send, rc);
       ctx->rc = rc;
-      close (sockets.send);
-      close (sockets.receive);
-      return rc;
+      if(!localMode){
+    	  closeSocket(&sockets,COM_ALL,"getProviderContext");
+      } else {
+    	  pthread_mutex_unlock(&resultsocketMutex);
+      }
+      _SFCB_RETURN(rc);
    }
 
    _SFCB_TRACE(1, ("--- Sending mgr request done"));
@@ -1039,8 +1096,9 @@ int getProviderContext(BinRequestContext * ctx, OperationHdr * ohdr)
    }
 
    if (!localMode) {
-      close (sockets.send);
-      close (sockets.receive);
+	  closeSocket(&sockets,COM_ALL,"getProviderContext");
+   } else {
+	   pthread_mutex_unlock(&resultsocketMutex);
    }
    _SFCB_RETURN(ctx->rc);
 }
@@ -1226,19 +1284,23 @@ static BinResponseHdr *intInvokeProvider(BinRequestContext * ctx,ComSockets sock
 BinResponseHdr *invokeProvider(BinRequestContext * ctx)
 {
    ComSockets sockets;
-   _SFCB_ENTER(TRACE_PROVIDERMGR | TRACE_CIMXMLPROC, "invokeProviders");
+   _SFCB_ENTER(TRACE_PROVIDERMGR | TRACE_CIMXMLPROC, "invokeProvider");
 
-   if (localMode) sockets=resultSockets;
+   if (localMode){
+	   pthread_mutex_lock(&resultsocketMutex);
+	   sockets=resultSockets;
+   }
    else sockets=getSocketPair("invokeProvider");
    
    BinResponseHdr *resp=intInvokeProvider(ctx, sockets);
     
    if (!localMode) {
-      close(sockets.receive);
-      close(sockets.send);
+	  closeSocket(&sockets,COM_ALL,"invokeProvider");
+   } else {
+	   pthread_mutex_unlock(&resultsocketMutex);
    }
    
-   return resp;
+   _SFCB_RETURN(resp);
 }
 
 BinResponseHdr **invokeProviders(BinRequestContext * binCtx, int *err,
@@ -1249,7 +1311,10 @@ BinResponseHdr **invokeProviders(BinRequestContext * binCtx, int *err,
    ComSockets sockets;
    int i;
 
-   if (localMode) sockets=resultSockets;
+   if (localMode){
+	   pthread_mutex_lock(&resultsocketMutex);
+	   sockets=resultSockets;
+   }
    else sockets=getSocketPair("invokeProvider");
    
    resp = malloc(sizeof(BinResponseHdr *) * (binCtx->pCount));
@@ -1276,8 +1341,9 @@ BinResponseHdr **invokeProviders(BinRequestContext * binCtx, int *err,
    }
    
    if (!localMode) {
-      close(sockets.receive);
-      close(sockets.send);
+	  closeSocket(&sockets,COM_ALL,"invokeProvider");
+   } else {
+	   pthread_mutex_unlock(&resultsocketMutex);
    }
    
    _SFCB_RETURN(resp);
@@ -1420,7 +1486,7 @@ static CMPIData localInvokeMethod(BinRequestContext * binCtx,
    InvokeMethodReq sreq = BINREQ(OPS_InvokeMethod, 5);
    OperationHdr req = { OPS_InvokeMethod, 1 };
    CMPIData data = { 0, CMPI_nullValue, {0} };
-   *out=NULL;
+   if (out) *out=NULL; /* out is used by getchildren and getassocs */
    BinResponseHdr *resp=NULL;
    
    sreq.in = setArgsMsgSegment(in);
@@ -1440,9 +1506,11 @@ static CMPIData localInvokeMethod(BinRequestContext * binCtx,
       resp->rc--;
       if (rc) rc->rc = resp->rc;
       if (resp->rc == CMPI_RC_OK) {
-         *out = relocateSerializedArgs(resp->object[0].data);
-         *out = (*out)->ft->clone(*out, NULL);
-         data=resp->rv;
+        if (out) {
+          *out = relocateSerializedArgs(resp->object[0].data);
+          *out = (*out)->ft->clone(*out, NULL);
+        }
+        data=resp->rv;
          // check for chars
       }
    }
@@ -1466,7 +1534,6 @@ int isChild(const char *ns, const char *parent, const char* child)
    BinRequestContext binCtx;
    OperationHdr req = { OPS_InvokeMethod, 1 };
    CMPIArgs *in = NewCMPIArgs(NULL);
-   CMPIArgs *out = NULL;
    CMPIStatus rc;
    int irc;
 
@@ -1479,7 +1546,7 @@ int isChild(const char *ns, const char *parent, const char* child)
    irc = _methProvider(&binCtx, &req);
 
    if (irc == MSG_X_PROVIDER) {
-      localInvokeMethod(&binCtx, path, "ischild", in, &out, &rc,0);
+      localInvokeMethod(&binCtx, path, "ischild", in, NULL, &rc,0);
       irc=(rc.rc==CMPI_RC_OK);
    }
    else irc=0;
@@ -1497,7 +1564,6 @@ static int startUpProvider(const char* ns, const char *name)
    BinRequestContext binCtx;
    OperationHdr req = { OPS_InvokeMethod, 1 };
    CMPIArgs *in = NewCMPIArgs(NULL);
-   CMPIArgs *out = NULL;
    CMPIStatus rc;
    int irc;
 
@@ -1509,7 +1575,7 @@ static int startUpProvider(const char* ns, const char *name)
    irc = _methProvider(&binCtx, &req);
 
    if (irc == MSG_X_PROVIDER) {
-      localInvokeMethod(&binCtx, path, "_startup", in, &out, &rc, 1);
+      localInvokeMethod(&binCtx, path, "_startup", in, NULL, &rc, 1);
       irc=(rc.rc==CMPI_RC_OK);
    }
    else irc=0;
