@@ -44,39 +44,22 @@
 
 #include <getopt.h>
 #include <syslog.h>
+#include <pwd.h>
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#ifdef HAVE_SLP
-#include "cimslp.h"
-static int startSLP = 1;
-#endif
-
-#ifdef LOCAL_CONNECT_ONLY_ENABLE
-// from httpAdapter.c
 int sfcBrokerPid=0;
-#endif // LOCAL_CONNECT_ONLY_ENABLE
-
-//#define USE_THREADS
-
-// for use by thread in startHttpd, startDbpd
-struct adapterThreadParams {
-  int argc;
-  char* argv;
-  int sslMode;
-  int sfcPid;
-};
 
 extern void setExFlag(unsigned long f);
 extern char *parseTarget(const char *target);
 extern UtilStringBuffer *instanceToString(CMPIInstance * ci, char **props);
-extern int init_sfcBroker(char *);
+extern int init_sfcBroker();
 extern CMPIBroker *Broker;
 extern void initProvProcCtl(int);
 extern void processTerminated(int pid);
-extern int httpDaemon(int argc, char *argv[], int sslMode, int pid);
+extern int httpDaemon(int argc, char *argv[], int sslMode);
 extern void processProviderMgrRequests();
 
 extern int stopNextProc();
@@ -88,7 +71,6 @@ extern void sunsetControl();
 extern void uninitGarbageCollector();
 
 extern TraceId traceIds[];
-extern int sfcBrokerPid;
 
 extern unsigned long exFlags;
 static int startHttp = 0;
@@ -167,51 +149,6 @@ static int stopNextAdapter()
    return 0;
 }
 
-
-#ifdef USE_THREADS
-// thread-based adapter functions
-
-static void addStartedThreadAdapter(pthread_t tid)
-{
- 
-   StartedThreadAdapter *sa=(StartedThreadAdapter*)malloc(sizeof(StartedThreadAdapter));
-
-   sa->stopped=0;
-   sa->tid=tid;
-   sa->next=lastStartedThreadAdapter;
-   lastStartedThreadAdapter=sa;
-}
-
-static int testStartedThreadAdapter(pthread_t tid, int *left) 
-{
-   StartedThreadAdapter *sa=lastStartedThreadAdapter;
-   int stopped=0;
-   
-   *left=0;
-   while (sa) {
-     if (pthread_equal(sa->tid, tid)) stopped=sa->stopped=1;
-      if (sa->stopped==0) (*left)++;
-      sa=sa->next;
-   }
-   return stopped;
-}         
-
-static int stopNextThreadAdapter()
-{
-   StartedThreadAdapter *sa=lastStartedThreadAdapter;
-   
-   while (sa) {
-      if (sa->stopped==0) {
-         sa->stopped=1;
-         pthread_exit(sa->tid);
-         return sa->tid;
-      }   
-      sa=sa->next;
-   }
-   return 0;
-}
-#endif
-
 static pthread_mutex_t sdMtx=PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t  sdCnd=PTHREAD_COND_INITIALIZER;
 static int stopping=0;
@@ -265,7 +202,8 @@ static void stopBroker(void *p)
    uninitSocketPairs();
    sunsetControl();
 //   uninitGarbageCollector();
-   
+   closeLogging();
+ 
    if (restartBroker) {
       char *emsg=strerror(errno);
       execvp("sfcbd",restartArgv);
@@ -406,35 +344,39 @@ static void handleSigAbort(int sig)
 
 
 #ifndef LOCAL_CONNECT_ONLY_ENABLE
-#ifdef USE_THREADS
-void* startHttpThread(void* params) {
-  struct adapterThreadParams* p = (struct adapterThreadParams*)params;
-  currentProc=getpid();
-
-  httpDaemon(p->argc, p->argv, p->sslMode, p->sfcPid);
-  closeSocket(&sfcbSockets,cRcv,"startHttpd");
-  closeSocket(&resultSockets,cAll,"startHttpd");
-
-}
-#endif
 
 static int startHttpd(int argc, char *argv[], int sslMode)
 {
-#ifdef USE_THREADS
-
    int pid,sfcPid=currentProc;
-   struct adapterThreadParams htparams = {argc, argv, sslMode, sfcPid};
-   pthread_t httpThread;
-   pthread_create(&httpThread, NULL, &startHttpThread, &htparams);
-   //newThread(&startHttpThread, &htparams, 0);
+   int httpSFCB,rc;
+   char *httpUser;
+   uid_t httpuid;
+   struct passwd *passwd;
 
-   pthread_yield();
-
-   addStartedThreadAdapter(httpThread);
-
-#else
-
-   int pid,sfcPid=currentProc;
+   // Get/check http user info
+   if (getControlBool("httpUserSFCB",&httpSFCB)) {
+     mlogf(M_ERROR,M_SHOW,"--- Error retrieving http user info from config file.\n");
+     exit(2);
+   } 
+   if (httpSFCB) {
+     // This indicates that we should use the SFCB user by default
+     httpuid = -1;
+   } else {
+     // Get the user specified in the config file
+     if (getControlChars("httpUser",&httpUser)) {
+        mlogf(M_ERROR,M_SHOW,"--- Error retrieving http user info from config file.\n");
+        exit(2);
+     } else {
+        errno=0;
+        passwd=getpwnam(httpUser);
+        if (passwd) {
+            httpuid=passwd->pw_uid;
+        } else {
+            mlogf(M_ERROR,M_SHOW,"--- Couldn't find http username %s requested in SFCB config file. Errno: %d\n",httpUser,errno);
+            exit(2);
+        }
+     }
+    }
 
    pid= fork();
    if (pid < 0) {
@@ -444,7 +386,18 @@ static int startHttpd(int argc, char *argv[], int sslMode)
    }
    if (pid == 0) {
       currentProc=getpid();
-      httpDaemon(argc, argv, sslMode, sfcPid);
+      if (httpuid != -1 ) {
+          // Set the real and effective uids
+          rc=setreuid(httpuid,httpuid);
+          if (rc == -1) {
+              mlogf(M_ERROR,M_SHOW,"--- Changing uid for http failed.\n");
+              exit(2);
+          }
+      }
+   
+      if (httpDaemon(argc, argv, sslMode)) {
+	kill(sfcPid, 3);          /* if port in use, shutdown */
+      }
       closeSocket(&sfcbSockets,cRcv,"startHttpd");
       closeSocket(&resultSockets,cAll,"startHttpd");
    }
@@ -452,7 +405,6 @@ static int startHttpd(int argc, char *argv[], int sslMode)
       addStartedAdapter(pid);
       return 0;
    }
-#endif
    return 0;
 }
 
@@ -485,16 +437,6 @@ static int startDbpd(int argc, char *argv[], int sslMode)
 
 #endif
 
-#ifdef HAVE_SLP
-
-static int startSLPAgent()
-{
-    slpAgent();
-    return 0;
-}
-
-#endif
-
 static void usage(int status)
 {
     if (status != 0)
@@ -519,6 +461,7 @@ static void usage(int status)
 		"                                 components to trace; ? lists the available",
 		"                                 components with their bitmask and exits",
 		" -v, --version                   output version information and exit",
+		" -i, --disable-repository-default-inst-prov To disable entry into the default provider",
 		"",
 		"For SBLIM package updates and additional information, please see",
 		"    the SBLIM homepage at http://sblim.sourceforge.net"
@@ -580,10 +523,11 @@ int main(int argc, char *argv[])
 	   { "syslog-level",     required_argument, 0,        'l' },
 	   { "trace-components", required_argument, 0,        't' },
 	   { "version",          no_argument,       0,        'v' },
+           { "disable-repository-default-inst-provider", no_argument,       0,        'i' },
 	   { 0, 0, 0, 0 }
        };
 
-   while ((c = getopt_long(argc, argv, "c:dhkst:vl:", long_options, 0)) != -1)
+   while ((c = getopt_long(argc, argv, "c:dhkst:vil:", long_options, 0)) != -1)
    {
        switch(c)
        {
@@ -628,6 +572,10 @@ int main(int argc, char *argv[])
 	   case 'v':
 	       version();
 
+           case 'i':
+               disableDefaultProvider=1;
+               break;
+
 	   case 'l':
             if (strcmp(optarg,"LOG_ERR")==0) {
                 syslogLevel=LOG_ERR;
@@ -645,7 +593,14 @@ int main(int argc, char *argv[])
 	       usage(3);
        }
    }
-   startLogging("sfcb",syslogLevel);
+
+   if (optind < argc)
+   {
+      fprintf(stderr,"SFCB not started: unrecognized config property %s\n",argv[optind]);
+      usage(1);
+   }
+
+   startLogging(syslogLevel);
 
    mlogf(M_INFO,M_SHOW,"--- %s V" sfcHttpDaemonVersion " started - %d\n", name, currentProc);
 
@@ -762,13 +717,13 @@ int main(int argc, char *argv[])
 	   "--- Max provider process number adjusted to %d\n", pSockets);
    }
 
-   if ((enableHttp && dSockets > 0) || (enableHttps && sSockets > 0) ) {
+   if ((enableHttp || enableHttps) && dSockets > 0) {
      startHttp = 1;
    }
    
    initSem(dSockets,sSockets,pSockets);
    initProvProcCtl(pSockets);
-   init_sfcBroker(NULL);
+   init_sfcBroker();
    initSocketPairs(pSockets,dSockets,sSockets);
 
    setSignal(SIGQUIT, handleSigquit,0);
@@ -783,10 +738,7 @@ int main(int argc, char *argv[])
    
 #ifndef LOCAL_CONNECT_ONLY_ENABLE
    if (startHttp) {
-      if (sslMode)
-         startHttpd(argc, argv,1);
-      if (!sslOMode)
-         startHttpd(argc, argv,0);
+     startHttpd(argc, argv, sslMode);
    }
 #endif // LOCAL_CONNECT_ONLY_ENABLE
    
@@ -801,13 +753,6 @@ int main(int argc, char *argv[])
 #endif	
 
 
-#ifdef HAVE_SLP
-   //Start SLP Agent
-   if (startSLP) {
-		startSLPAgent();
-   }
-#endif
-   
    setSignal(SIGSEGV, handleSigSegv,SA_ONESHOT);
    setSignal(SIGCHLD, handleSigChld,0);
    
